<?php

namespace Amazingcat\Labels\Plugin\Block\Product;

use Magento\Catalog\Block\Product\Image as ImageBlock;
use Magento\Catalog\Block\Product\ImageFactory;
use Magento\Catalog\Model\Product as ProductModel;
use Magento\Catalog\Model\View\Asset\ImageFactory as AssetImageFactory;
use Magento\Catalog\Model\Product;
use Magento\Catalog\Model\Product\Image\ParamsBuilder;
use Magento\Catalog\Model\View\Asset\PlaceholderFactory;
use Magento\Framework\EntityManager\EventManager;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\View\ConfigInterface;
use Magento\Catalog\Helper\Image as ImageHelper;
use Amazingcat\Labels\Model\ResourceModel\DefaultLabel\CollectionFactory as LabelCollectionFactory;
use Magento\Catalog\Model\ProductRepository;
use Magento\CatalogRule\Model\ResourceModel\Rule;
use Magento\CatalogRule\Model\ResourceModel\Rule\CollectionFactory;
use Magento\Customer\Model\Session;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Model\StoreManagerInterface;
use \Psr\Log\LoggerInterface;

/**
 * Class ImageFactoryPlugin
 * @package Amazingcat\Labels\Plugin\Block\Product
 */
class ImageFactoryPlugin extends ImageFactory
{
    /**
     * @var ConfigInterface
     */
    private $presentationConfig;

    /**
     * @var AssetImageFactory
     */
    private $viewAssetImageFactory;

    /**
     * @var ParamsBuilder
     */
    private $imageParamsBuilder;

    /**
     * @var ObjectManagerInterface
     */
    private $objectManager;

    /**
     * @var PlaceholderFactory
     */
    private $viewAssetPlaceholderFactory;

    /**
     * @var Rule
     */
    protected $rule;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var CollectionFactory
     */
    protected $ruleCollectionFactory;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * @var Session
     */
    protected $session;

    /**
     * @var ProductRepository
     */
    protected $productRepository;

    /**
     * @var LabelCollectionFactory
     */
    protected $defaultLabelCollectionFactory;

    /**
     * @var EventManager
     */
    protected $_eventManager;

    /**
     * ImageFactory constructor.
     * @param ObjectManagerInterface $objectManager
     * @param ConfigInterface $presentationConfig
     * @param AssetImageFactory $viewAssetImageFactory
     * @param PlaceholderFactory $viewAssetPlaceholderFactory
     * @param ParamsBuilder $imageParamsBuilder
     * @param Rule $rule
     * @param CollectionFactory $ruleCollectionFactory
     * @param StoreManagerInterface $storeManager
     * @param LoggerInterface $logger
     * @param Session $session
     * @param ProductRepository $productRepository
     * @param LabelCollectionFactory $defaultLabelCollectionFactory
     * @param EventManager $eventManager
     */
    public function __construct(
        ObjectManagerInterface $objectManager,
        ConfigInterface $presentationConfig,
        AssetImageFactory $viewAssetImageFactory,
        PlaceholderFactory $viewAssetPlaceholderFactory,
        ParamsBuilder $imageParamsBuilder,
        Rule $rule,
        CollectionFactory $ruleCollectionFactory,
        StoreManagerInterface $storeManager,
        LoggerInterface $logger,
        Session $session,
        ProductRepository $productRepository,
        LabelCollectionFactory $defaultLabelCollectionFactory,
        EventManager $_eventManager
    ) {
        parent::__construct(
            $objectManager,
            $presentationConfig,
            $viewAssetImageFactory,
            $viewAssetPlaceholderFactory,
            $imageParamsBuilder
        );
        $this->objectManager = $objectManager;
        $this->presentationConfig = $presentationConfig;
        $this->viewAssetImageFactory = $viewAssetImageFactory;
        $this->viewAssetPlaceholderFactory = $viewAssetPlaceholderFactory;
        $this->imageParamsBuilder = $imageParamsBuilder;

        $this->rule = $rule;
        $this->ruleCollectionFactory = $ruleCollectionFactory;
        $this->storeManager = $storeManager;
        $this->logger = $logger;
        $this->session = $session;
        $this->productRepository = $productRepository;
        $this->defaultLabelCollectionFactory = $defaultLabelCollectionFactory;
        $this->_eventManager = $_eventManager;
    }

    /**
     * Retrieve image custom attributes for HTML element
     *
     * @param array $attributes
     * @return string
     */
    private function getStringCustomAttributes(array $attributes): string
    {
        $result = [];
        foreach ($attributes as $name => $value) {
            $result[] = $name . '="' . $value . '"';
        }

        return !empty($result) ? implode(' ', $result) : '';
    }

    /**
     * Calculate image ratio
     *
     * @param $width
     * @param $height
     * @return float
     */
    private function getRatio(int $width, int $height): float
    {
        if ($width && $height) {
            return $height / $width;
        }

        return 1.0;
    }

    /**
     * @param Product $product
     *
     * @param string $imageType
     * @return string
     */
    private function getLabel(Product $product, string $imageType): string
    {
        $label = $product->getData($imageType . '_' . 'label');
        if (empty($label)) {
            $label = $product->getName();
        }

        return (string)$label;
    }

    /**
     * Create image block from product
     * @param ImageFactory $subject
     * @param $result
     * @param Product $product
     * @param string $imageId
     * @param array|null $attributes
     * @return ImageBlock
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function afterCreate(ImageFactory $subject, $result, Product $product, string $imageId, array $attributes = null): ImageBlock
    {
        $viewImageConfig = $this->presentationConfig->getViewConfig()->getMediaAttributes(
            'Magento_Catalog',
            ImageHelper::MEDIA_TYPE_CONFIG_NODE,
            $imageId
        )
        ;

        $imageMiscParams = $this->imageParamsBuilder->build($viewImageConfig);
        $originalFilePath = $product->getData($imageMiscParams['image_type']);

        if ($originalFilePath === null || $originalFilePath === 'no_selection') {
            $imageAsset = $this->viewAssetPlaceholderFactory->create(
                [
                    'type' => $imageMiscParams['image_type'],
                ]
            );
        } else {
            $imageAsset = $this->viewAssetImageFactory->create(
                [
                    'miscParams' => $imageMiscParams,
                    'filePath'   => $originalFilePath,
                ]
            );
        }

        $data = [
            'data' => [
                'template'          => 'Amazingcat_Labels::product/image_with_borders.phtml',
                'image_url'         => $imageAsset->getUrl(),
                'width'             => $imageMiscParams['image_width'],
                'height'            => $imageMiscParams['image_height'],
                'label'             => $this->getLabel($product, $imageMiscParams['image_type']),
                'ratio'             => $this->getRatio($imageMiscParams['image_width'], $imageMiscParams['image_height']),
                'custom_attributes' => $this->getStringCustomAttributes($attributes),
                'product_id'        => $product->getId(),
                'product_label'     => $this->getProductLabelImageUrl($product),
            ],
        ];

        return $this->objectManager->create(ImageBlock::class, $data);
    }

    /**
     * @param ProductModel $product
     * @return array|null
     * @throws NoSuchEntityException
     */
    protected function getProductLabelImageUrl(ProductModel $product)
    {
        $productRules = $this->rule->getRulesFromProduct(
            time(),
            $this->storeManager->getStore()->getWebsiteId(),
            $this->session->getCustomerGroupId(),
            $product->getId()
        );

        if (empty($productRules)) {
            return null;
        }

        $response = [
            'label'    => '',
            'position' => 0,
        ];

        try {
            foreach ($productRules as $productRule) {
                $ruleRecord = $this->ruleCollectionFactory->create()
                    ->addFieldToFilter('rule_id', $productRule['rule_id'])
                    ->addFieldToFilter('is_active', 1)
                    ->getFirstItem()
                ;

                if ($ruleRecord->getRuleLabel()) {
                    $ruleBadge = $ruleRecord->getRuleLabel();
                }
            }

            $productLabel = $product->getProductLabel();

            if (!empty($ruleBadge)) {
                $response['label'] = $ruleBadge;

                $this->_eventManager->dispatch('amazingcat_labels_render_label', ['label' => $response]);

                return $response;
            } elseif (!empty($productLabel) && $productLabel != 'no_selection') {
                $productLabel = $product->getProductLabel();
                $response['label'] = '/pub/media/catalog/product' . $productLabel;

                $this->_eventManager->dispatch('amazingcat_labels_render_label', ['label' => $response]);

                return $response;
            } else {
                $productReloaded = $this->productRepository->getById($product->getId(), false, null, true);
                $defaultLabel = $productReloaded->getDefaultLabel();

                if ($defaultLabel == 0) {
                    return null;
                }

                $collection = $this->defaultLabelCollectionFactory->create();
                $defaultLabel = $collection
                    ->addFieldToFilter('id', $defaultLabel)
                    ->getFirstItem()
                ;

                if (!empty($url = $defaultLabel->getImagePath())) {
                    $imagePath = '/pub' . $defaultLabel->getImagePath();
                    $response = [
                        'label'    => $imagePath,
                        'position' => $defaultLabel->getPosition(),
                    ];
                } else {
                    return null;
                }

                $this->_eventManager->dispatch('amazingcat_labels_render_label', ['label' => $response]);

                return $response;
            }
        } catch (NoSuchEntityException $e) {
            $this->logger->critical($e->getFile() . ": " . $e->getMessage());

            return null;
        } catch (\Throwable $e) {
            $this->logger->critical($e->getFile() . ": " . $e->getMessage());

            return null;
        }
    }
}